//+------------------------------------------------------------------+
//|                                               RSI_divergence.mq4 |
//|                                                          k-matsu |
//|                               http://www.age.jp/~k-matsu/FX/MT4/ |
//+------------------------------------------------------------------+

// version 1.00  2010/03/27
//         1.01  2010/03/31 bug fix
//         1.02  2010/04/06 bug fix(?)

#property copyright "k-matsu"
#property link      "http://www.age.jp/~k-matsu/FX/MT4/"

#property indicator_separate_window
//#property indicator_minimum 0
//#property indicator_maximum 100
#property indicator_buffers 2
#property indicator_color1 BlueViolet
#property indicator_color2 BlueViolet
#property  indicator_level1  70
#property  indicator_level2  30

//---- input parameters
extern int       RSI1Period=14;
extern int       RSI1Price=PRICE_CLOSE;
extern int       RSI2Period=14;
extern int       RSI2Price=PRICE_CLOSE;
extern int       SwingMargin=4;
extern int       RSIMargin=2;
extern bool      PeakConvergence=false;
extern bool      TroughDivergence=false;
extern color     DivergenceColor=Red;
extern color     ConvergenceColor=Aqua;
extern bool      GuideLine=true;
extern color     GuidePeakColor=Aqua;
extern color     GuideTroughColor=Red;
extern int       GuidePeakStyle=STYLE_DOT;
extern int       GuideTroughStyle=STYLE_DOT;
//---- buffers
double RSI1Buffer[];
double RSI2Buffer[];
double PeakBuffer[];
double TroughBuffer[];
//---- indicator name
string shortname_base="RSI_divergence";
string shortname;
//----

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   // sweep lines
   deinit();

//---- indicators
   IndicatorBuffers(4);
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,RSI1Buffer);
   SetIndexLabel(0, "RSI1("+RSI1Period+")");
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,RSI2Buffer);
   SetIndexLabel(1, "RSI2("+RSI2Period+")");
   SetIndexBuffer(2,PeakBuffer);
   SetIndexBuffer(3,TroughBuffer);

   shortname = shortname_base+" ("+RSI1Period+","+RSI2Period+")";
   IndicatorShortName(shortname);

   // parameter check
   if (RSI1Price < 0) RSI1Price = 0;
   if (RSI1Price > 6) RSI1Price = 6;
   if (RSI2Price < 0) RSI2Price = 0;
   if (RSI2Price > 6) RSI2Price = 6;
   if (SwingMargin < 1) SwingMargin = 1;
   if (RSIMargin < 1) RSIMargin = 1;
   if (GuidePeakStyle < 0) GuidePeakStyle = 0;
   if (GuidePeakStyle > 4) GuidePeakStyle = 4;
   if (GuideTroughStyle < 0) GuideTroughStyle = 0;
   if (GuideTroughStyle > 4) GuideTroughStyle = 4;
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
     int shortname_length=StringLen(shortname);
     for(int i = ObjectsTotal() - 1; i >= 0; i--) {
       string label = ObjectName(i);
       if(StringSubstr(label, 0, shortname_length) == shortname)
         ObjectDelete(label);
     }
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
    int limit, limitbase, limitRSI1, limitRSI2;
    int counted_bars = IndicatorCounted();
    int prevpeakidx, prevtroughidx;
    if (counted_bars > 0) counted_bars--;
    int RSIidx, prevRSIidx;
    bool found=false;
    datetime peaktime, prevpeaktime, troughtime, prevtroughtime, RSItime, prevRSItime;
    int window=WindowOnDropped();
    if (window <= 0) {
      window=WindowFind(shortname);
    }

    limitbase = Bars - counted_bars;
    for (int i = 0; i< limitbase; i++) {
      PeakBuffer[i] = EMPTY_VALUE;
      TroughBuffer[i] = EMPTY_VALUE;
    }

    limitRSI1 = Bars - MathMax(counted_bars, RSI1Period);
    for (i = 0; i< limitRSI1; i++) {
      RSI1Buffer[i] = iRSI(NULL, 0, RSI1Period, RSI1Price, i);
    }

    limitRSI2 = Bars - MathMax(counted_bars, RSI2Period);
    for (i = 0; i< limitRSI2; i++) {
      RSI2Buffer[i] = iRSI(NULL, 0, RSI2Period, RSI2Price, i);
    }

  // search peak and divergence
    prevpeakidx = 1; prevRSIidx = 0; limitRSI1 = Bars-MathMax(RSI1Period, SwingMargin);
    for (limit = 3; limit < limitRSI1 && PeakBuffer[limit] == EMPTY_VALUE; limit++) {}
    if (limit > SwingMargin) {
      for (i = 2; i< limit && High[i] == High[1]; i++) {}
      if (High[i] < High[1]) {
        i = searchNextTroughIdx(MODE_HIGH, i, limit, 1) + 1;
      }
      while (i < limit) {
        found=false;
        i = searchNextPeakIdx(MODE_HIGH, i, limit, SwingMargin);
        if (i - prevpeakidx >= SwingMargin) {
          if (PeakBuffer[i] == EMPTY_VALUE) {
            if (iHighest(NULL, 0, MODE_HIGH, SwingMargin*2+1, i-SwingMargin) == i) {
              found = true;
              PeakBuffer[i] = High[i];
            }
          } else {
            found = true;
          }
          if (found) {
            peaktime=Time[i];
            RSIidx = ArrayMaximum(RSI1Buffer, RSIMargin*2+1, i-RSIMargin);
            RSItime=Time[RSIidx];
            if (prevRSIidx > 0) {
              prevpeaktime=Time[prevpeakidx];
              prevRSItime=Time[prevRSIidx];
              if (High[i] < High[prevpeakidx] && RSI1Buffer[RSIidx] > RSI1Buffer[prevRSIidx]) {
                //divergence!
                createLine(StringConcatenate("pr1",peaktime,",",prevpeaktime), 0, peaktime, High[i], prevpeaktime, High[prevpeakidx], DivergenceColor);
                createLine(StringConcatenate("RSI1",RSItime,",",prevRSItime), window, RSItime, RSI1Buffer[RSIidx], prevRSItime, RSI1Buffer[prevRSIidx], DivergenceColor);
              }
              if (PeakConvergence) {
                if (High[i] > High[prevpeakidx] && RSI1Buffer[RSIidx] < RSI1Buffer[prevRSIidx]) {
                  //peak convergence!
                  createLine(StringConcatenate("pr1",peaktime,",",prevpeaktime), 0, peaktime, High[i], prevpeaktime, High[prevpeakidx], ConvergenceColor);
                 createLine(StringConcatenate("RSI1",RSItime,",",prevRSItime), window, RSItime, RSI1Buffer[RSIidx], prevRSItime, RSI1Buffer[prevRSIidx], ConvergenceColor);
                }
              }
            } else {
              if (GuideLine) {
                createGuideLine("PricePeak", 0, peaktime, High[i], GuidePeakColor, GuidePeakStyle);
                createGuideLine("RSIPeak", window, RSItime, RSI1Buffer[RSIidx], GuidePeakColor, GuidePeakStyle);
              }
            }
            prevpeakidx = i;
            prevRSIidx = RSIidx;
          }
        }
        i = searchNextTroughIdx(MODE_HIGH, i+1, limit, 1);
        i++;
      }
    }


  // search trough and convergence
    prevtroughidx = 1; prevRSIidx = 0; limitRSI2 = Bars-MathMax(RSI2Period, SwingMargin);
    for (limit = 3; limit < limitRSI2 && TroughBuffer[limit] == EMPTY_VALUE; limit++) {}
    if (limit > SwingMargin) {
      for (i = 2; i< limit && Low[i] == Low[1]; i++) {}
      if (Low[i] > Low[1]) {
        i = searchNextPeakIdx(MODE_LOW, i, limit, 1) + 1;
      }
      while (i < limit) {
        found = false;
        i = searchNextTroughIdx(MODE_LOW, i, limit, SwingMargin);
        if (i - prevtroughidx >= SwingMargin) {
          if (TroughBuffer[i] == EMPTY_VALUE) {
            if (iLowest(NULL, 0, MODE_LOW, SwingMargin*2+1, i-SwingMargin) == i) {
              found = true;
              TroughBuffer[i] = Low[i];
            }
          } else {
            found = true;
          }
          if (found) {
            troughtime=Time[i];
            RSIidx = ArrayMinimum(RSI2Buffer, RSIMargin*2+1, i-RSIMargin);
            RSItime=Time[RSIidx];
            if (prevRSIidx > 0) {
              prevtroughtime=Time[prevtroughidx];
              prevRSItime=Time[prevRSIidx];
              if (Low[i] > Low[prevtroughidx] && RSI2Buffer[RSIidx] < RSI2Buffer[prevRSIidx]) {
                //convergence!
                createLine(StringConcatenate("pr2",troughtime,",",prevtroughtime), 0, troughtime, Low[i], prevtroughtime, Low[prevtroughidx], ConvergenceColor);
                createLine(StringConcatenate("RSI2",RSItime+",",prevRSItime), window, RSItime, RSI2Buffer[RSIidx], prevRSItime, RSI2Buffer[prevRSIidx], ConvergenceColor);
              }
              if (TroughDivergence) {
                if (Low[i] < Low[prevtroughidx] && RSI2Buffer[RSIidx] > RSI2Buffer[prevRSIidx]) {
                  // trough dinvergence!
                  createLine(StringConcatenate("pr2",troughtime,",",prevtroughtime), 0, troughtime, Low[i], prevtroughtime, Low[prevtroughidx], DivergenceColor);
                  createLine(StringConcatenate("RSI2",RSItime,",",prevRSItime), window, RSItime, RSI2Buffer[RSIidx], prevRSItime, RSI2Buffer[prevRSIidx], DivergenceColor);
                }
              }
            } else {
              if (GuideLine) {
                createGuideLine("PriceTrough", 0, troughtime, Low[i], GuideTroughColor, GuideTroughStyle);
                createGuideLine("RSITrough", window, RSItime, RSI2Buffer[RSIidx], GuideTroughColor, GuideTroughStyle);
              }
            }
            prevtroughidx = i;
            prevRSIidx = RSIidx;
          }
        }
        i = searchNextPeakIdx(MODE_LOW, i+1, limit, 1);
        i++;
      }
    }
   
//----
   return(0);
  }
//+------------------------------------------------------------------+

// subfunction

// need call after searchNextTroughIdx
int searchNextPeakIdx(int mode, int startidx, int limit, int margin)
{
  int retidx, nextstartidx = startidx, prevpeakidx = nextstartidx, count;

  count = margin*2+1;  
  if (nextstartidx+count > limit) count = limit-nextstartidx+1;

  retidx = iHighest(NULL, 0, mode, count, nextstartidx);
  while (retidx > prevpeakidx && retidx < limit) {
    prevpeakidx = retidx;

    nextstartidx =prevpeakidx-margin;
    if (nextstartidx <= startidx) nextstartidx = startidx+1;

    count = margin*2+1;
    if (nextstartidx+count >= limit) count = limit-nextstartidx+1;

    retidx = iHighest(NULL, 0, mode, count, nextstartidx);
  }

  return(retidx);
}

// need call after searchNextPeakIdx
int searchNextTroughIdx(int mode, int startidx, int limit, int margin)
{
  int retidx, nextstartidx = startidx, prevtroughidx = nextstartidx, count;

  count = margin*2+1;
  if (nextstartidx+count > limit) count = limit-nextstartidx+1;

  retidx = iLowest(NULL, 0, mode, count, nextstartidx);
  while (retidx > prevtroughidx && retidx < limit) {
    prevtroughidx = retidx;

    nextstartidx =prevtroughidx-margin;
    if (nextstartidx <= startidx) nextstartidx = startidx+1;

    count = margin*2+1;
    if (nextstartidx+count >= limit) count = limit-nextstartidx+1;

    retidx = iLowest(NULL, 0, mode, count, nextstartidx);  
  }

  return(retidx);
}


int createLine(string subname, int window, datetime time1, double price1, datetime time2, double price2, color linecolor)
{
  string objectname = shortname+subname;
  if (ObjectFind(objectname)==-1) {
    ObjectCreate(objectname, OBJ_TREND, window, time1, price1, time2, price2);
  }
  ObjectSet(objectname, OBJPROP_RAY,    false);
  ObjectSet(objectname, OBJPROP_TIME1,  time1);
  ObjectSet(objectname, OBJPROP_PRICE1, price1);
  ObjectSet(objectname, OBJPROP_TIME2,  time2);
  ObjectSet(objectname, OBJPROP_PRICE2, price2);
  ObjectSet(objectname, OBJPROP_COLOR, linecolor);
  return(0);
}

int createGuideLine(string subname, int window, datetime time, double price, color linecolor, int linestyle)
{
  string objectname = shortname+subname;
  ObjectCreate(objectname, OBJ_TREND, window, time, price, Time[0], price);
  ObjectSet(objectname, OBJPROP_RAY,    true);
  ObjectSet(objectname, OBJPROP_TIME1,  time);
  ObjectSet(objectname, OBJPROP_PRICE1, price);
  ObjectSet(objectname, OBJPROP_TIME2,  Time[0]);
  ObjectSet(objectname, OBJPROP_PRICE2, price);
  ObjectSet(objectname, OBJPROP_COLOR, linecolor);
  ObjectSet(objectname, OBJPROP_STYLE,  linestyle);
  return(0);
}

